@@ -1,7 +1,7 @@
 /* enchant
  * Copyright (C) 2003 Yaacov Zamir
  * Copyright (C) 2004 Dom Lachowicz
- * Copyright (C) 2017-2024 Reuben Thomas
+ * Copyright (C) 2017-2025 Reuben Thomas
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -46,6 +46,8 @@
 
 #define HSPELL_MAIN_SUFFIX ".wgz"
 
+static EnchantProvider *provider;
+
 /**
  * convert struct corlist to **char
  * the **char must be g_freed
@@ -53,19 +55,14 @@
 static gchar **
 corlist2strv (struct corlist *cl, size_t nb_sugg)
 {
-	char **sugg_arr = NULL;
-	if (nb_sugg > 0)
-		{
-			sugg_arr = g_new0 (char *, nb_sugg + 1);
-			for (size_t i = 0; i < nb_sugg; i++)
-				{
-					gsize len;
-					const char *sugg = corlist_str (cl, i);
-					if (sugg)
-						sugg_arr[i] = g_convert (sugg,
-									 strlen (sugg),
-									 "utf-8", "iso8859-8", NULL, &len, NULL);
-				}
+	char **sugg_arr = g_new0 (char *, nb_sugg + 1);
+	if (sugg_arr)
+		for (size_t i = 0; i < nb_sugg; i++) {
+			gsize len;
+			const char *sugg = corlist_str (cl, i);
+			sugg_arr[i] = g_convert (sugg,
+						 strlen (sugg),
+						 "utf-8", "iso8859-8", NULL, &len, NULL);
 		}
 
 	return sugg_arr;
@@ -87,7 +84,10 @@ hspell_dict_check (EnchantDict * me, const char *const word, size_t len)
 {
 	struct dict_radix *hspell_dict = (struct dict_radix *)me->user_data;
 	char *iso_word = hspell_convert_to_iso8859_8 (me, word, len);
-	g_return_val_if_fail (iso_word, -1);
+	if (iso_word == NULL) {
+		g_warning ("%s: Can't convert word to iso8859-8", __FUNCTION__);
+		return -1;
+	}
 
 	/* check */
 	int preflen;
@@ -108,7 +108,10 @@ hspell_dict_suggest (EnchantDict * me, const char *const word,
 {
 	struct dict_radix *hspell_dict = (struct dict_radix *)me->user_data;
 	char *iso_word = hspell_convert_to_iso8859_8 (me, word, len);
-	g_return_val_if_fail (iso_word, NULL);
+	if (iso_word == NULL) {
+		g_warning ("%s: Can't convert word to iso8859-8", __FUNCTION__);
+		return NULL;
+	}
 
 	/* get suggestions */
 	struct corlist cl;
@@ -135,9 +138,9 @@ hspell_provider_enum_dict_files (const char * const directory)
 			if (utf8_entry && g_strrstr (utf8_entry, HSPELL_MAIN_SUFFIX) != NULL) {
 				char * dic = g_build_filename(directory, utf8_entry, NULL);
 				char * desc_file = g_strconcat(dic, ".desc", NULL);
-				if (g_file_test(desc_file, G_FILE_TEST_EXISTS) != 0) {
+				if (desc_file && g_file_test(desc_file, G_FILE_TEST_EXISTS) != 0)
 					out_dicts = g_slist_append (out_dicts, dic);
-				} else
+				else
 					g_free (dic);
 				g_free (desc_file);
 			}
@@ -182,13 +185,14 @@ hspell_provider_request_dict (EnchantProvider * me, const char *const tag)
 	struct dict_radix *hspell_dict = NULL;
 	int dict_flag = hspell_init (&hspell_dict, HSPELL_OPT_DEFAULT);
 
-	if (dict_flag != 0 || !hspell_dict)
-		{
-			enchant_provider_set_error (me, "cannot get requested dictionary");
-			return NULL;
-		}
+	if (dict_flag != 0 || !hspell_dict) {
+		enchant_provider_set_error (me, "cannot get requested dictionary");
+		return NULL;
+	}
 
-	EnchantDict *dict = enchant_broker_new_dict (me->owner);
+	EnchantDict *dict = enchant_dict_new (provider, tag);
+	if (dict == NULL)
+		return NULL;
 	dict->user_data = (void *) hspell_dict;
 	dict->check = hspell_dict_check;
 	dict->suggest = hspell_dict_suggest;
@@ -214,21 +218,22 @@ hspell_provider_list_dicts (EnchantProvider * me, size_t * out_n_dicts)
 	GSList * dict_files = hspell_provider_enum_dict_files (user_dict_dir);
 	guint n_user_dicts = g_slist_length (dict_files);
 	char ** out_list = g_new0 (char *, n_user_dicts + 3);
-	for (guint i = 0; i < n_user_dicts; i++) {
-		gchar * dict_file = g_slist_nth_data (dict_files, i);
-		gchar * basename = g_path_get_basename (dict_file);
-		gchar * tag_end = g_strrstr (basename, HSPELL_MAIN_SUFFIX);
-		if (tag_end != NULL) {
-			*tag_end = '\0';
-			out_list[(*out_n_dicts)++] = basename;
-		} else
-			g_free (basename);
-	}
+	if (out_list)
+		for (guint i = 0; i < n_user_dicts; i++) {
+			gchar * dict_file = g_slist_nth_data (dict_files, i);
+			gchar * basename = g_path_get_basename (dict_file);
+			gchar * tag_end = g_strrstr (basename, HSPELL_MAIN_SUFFIX);
+			if (tag_end != NULL) {
+				*tag_end = '\0';
+				out_list[(*out_n_dicts)++] = basename;
+			} else
+				g_free (basename);
+		}
 	g_free (user_dict_dir);
 	g_slist_free_full (dict_files, g_free);
 
 	const char * dictionary_path = hspell_get_dictionary_path();
-	if(dictionary_path && *dictionary_path && g_file_test (dictionary_path, G_FILE_TEST_EXISTS)) {
+	if(out_list && dictionary_path && *dictionary_path && g_file_test (dictionary_path, G_FILE_TEST_EXISTS)) {
 		out_list[(*out_n_dicts)++] = g_strdup ("he");
 		out_list[(*out_n_dicts)++] = g_strdup ("he_IL");
 	}
@@ -253,7 +258,7 @@ EnchantProvider *init_enchant_provider (void);
 EnchantProvider *
 init_enchant_provider (void)
 {
-	EnchantProvider *provider = enchant_provider_new ();
+	provider = enchant_provider_new ();
 	provider->request_dict = hspell_provider_request_dict;
 	provider->dispose_dict = hspell_provider_dispose_dict;
 	provider->identify = hspell_provider_identify;
