@@ -2,6 +2,7 @@
  * Copyright (C) 2003,2004 Dom Lachowicz
  *               2006-2007 Harri Pitkänen <hatapitk@iki.fi>
  *               2006 Anssi Hannula <anssi.hannula@gmail.com>
+ *               2017-2025 Reuben Thomas
  *               2017 Børre Gaup <borre.gaup@uit.no>
  *
  * This library is free software; you can redistribute it and/or
@@ -11,7 +12,7 @@
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
@@ -46,6 +47,8 @@
  * https://voikko.sourceforge.net/
  */
 
+static EnchantProvider *provider;
+
 static int
 voikko_dict_check (EnchantDict * me, const char *const word, size_t len)
 {
@@ -69,14 +72,15 @@ voikko_dict_suggest (EnchantDict * me, const char *const word,
 	free(word_nul);
 	if (voikko_sugg_arr == NULL)
 		return NULL;
-	for (*out_n_suggs = 0; voikko_sugg_arr[*out_n_suggs] != NULL; (*out_n_suggs)++);
+	for (*out_n_suggs = 0; voikko_sugg_arr[*out_n_suggs] != NULL; (*out_n_suggs)++)
+		;
 
-	char **sugg_arr = calloc(sizeof (char *), *out_n_suggs + 1);
-	if (sugg_arr == NULL)
-		return NULL;
-	for (size_t i = 0; i < *out_n_suggs; i++) {
-		sugg_arr[i] = strdup (voikko_sugg_arr[i]);
-	}
+	char **sugg_arr = g_new0(char *, *out_n_suggs + 1);
+	if (sugg_arr)
+		for (size_t i = 0; i < *out_n_suggs; i++)
+			sugg_arr[i] = strdup (voikko_sugg_arr[i]);
+	else
+		*out_n_suggs = 0;
 	voikkoFreeCstrArray (voikko_sugg_arr);
 	return sugg_arr;
 }
@@ -85,50 +89,43 @@ static void
 voikko_provider_dispose_dict (EnchantProvider * me _GL_UNUSED, EnchantDict * dict)
 {
 	voikkoTerminate((struct VoikkoHandle *)dict->user_data);
-	free (dict);
 }
 
 static char **
 voikko_provider_list_dicts (EnchantProvider * me, size_t * out_n_dicts)
 {
-	size_t i;
-	char ** out_list = NULL;
 	*out_n_dicts = 0;
-	char * user_dict_dir = enchant_get_user_dict_dir (me);
+	char * user_dict_dir = enchant_provider_get_user_dict_dir (me);
 	char ** voikko_langs = voikkoListSupportedSpellingLanguages (user_dict_dir);
 	g_free (user_dict_dir);
 
-	for (i = 0; voikko_langs[i] != NULL; i++) {
+	for (size_t i = 0; voikko_langs[i] != NULL; i++)
 		(*out_n_dicts)++;
-	}
 
-	if (*out_n_dicts) {
-		out_list = calloc (*out_n_dicts + 1, sizeof (char *));
-		if (out_list != NULL)
-			for (i = 0; i < *out_n_dicts; i++) {
-				out_list[i] = strdup (voikko_langs[i]);
-			}
-	}
+	char ** out_list = g_new0 (char *, *out_n_dicts + 1);
+	if (out_list)
+		for (size_t i = 0; i < *out_n_dicts; i++)
+			out_list[i] = strdup (voikko_langs[i]);
+	else
+		*out_n_dicts = 0;
 	voikkoFreeCstrArray(voikko_langs);
-
 	return out_list;
 }
 
 static int
-voikko_provider_dictionary_exists (struct str_enchant_provider * me,
+voikko_provider_dictionary_exists (EnchantProvider * me,
 				   const char *const tag)
 {
-	char * user_dict_dir = enchant_get_user_dict_dir (me);
+	char * user_dict_dir = enchant_provider_get_user_dict_dir (me);
 	char ** voikko_langs = voikkoListSupportedSpellingLanguages (user_dict_dir);
 	g_free (user_dict_dir);
 
 	int exists = 0;
-	for (size_t i = 0; voikko_langs[i] != NULL; i++) {
+	for (size_t i = 0; voikko_langs[i] != NULL; i++)
 		if (strncmp (tag, voikko_langs[i], strlen (tag)) == 0) {
 			exists = 1;
 			break;
 		}
-	}
 	voikkoFreeCstrArray(voikko_langs);
 
 	return exists;
@@ -139,11 +136,10 @@ voikko_provider_request_dict (EnchantProvider * me, const char *const tag)
 {
 	const char * voikko_error;
 
-	if (!voikko_provider_dictionary_exists (me, tag)) {
+	if (!voikko_provider_dictionary_exists (me, tag))
 		return NULL;
-	}
 
-	char * user_dict_dir = enchant_get_user_dict_dir (me);
+	char * user_dict_dir = enchant_provider_get_user_dict_dir (me);
 	struct VoikkoHandle *voikko_handle = voikkoInit (&voikko_error, tag, user_dict_dir);
 	g_free (user_dict_dir);
 	if (voikko_handle == NULL) {
@@ -151,7 +147,7 @@ voikko_provider_request_dict (EnchantProvider * me, const char *const tag)
 		return NULL;
 	}
 
-	EnchantDict *dict = calloc (sizeof (EnchantDict), 1);
+	EnchantDict *dict = enchant_dict_new (provider, tag);
 	if (dict == NULL)
 		return NULL;
 	dict->user_data = (void *)voikko_handle;
@@ -161,12 +157,6 @@ voikko_provider_request_dict (EnchantProvider * me, const char *const tag)
 	return dict;
 }
 
-static void
-voikko_provider_dispose (EnchantProvider * me)
-{
-	free (me);
-}
-
 static const char *
 voikko_provider_identify (EnchantProvider * me _GL_UNUSED)
 {
@@ -184,10 +174,9 @@ EnchantProvider *init_enchant_provider (void);
 EnchantProvider *
 init_enchant_provider (void)
 {
-	EnchantProvider *provider = calloc (sizeof (EnchantProvider), 1);
+	provider = enchant_provider_new ();
 	if (provider == NULL)
 		return NULL;
-	provider->dispose = voikko_provider_dispose;
 	provider->request_dict = voikko_provider_request_dict;
 	provider->dispose_dict = voikko_provider_dispose_dict;
 	provider->dictionary_exists = voikko_provider_dictionary_exists;
